# OnlineCourse Product for Zope
# Copyright (C) 2001 Maurice Davis <mjd@networklogic.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# See file COPYING.txt
#
#
__doc__ = """Assignment Implementation for OnlineCourse Product."""

__version__ = '0.1.0'

from AccessControl import ClassSecurityInfo
from AccessControl.Role import RoleManager

from Globals import InitializeClass
from Globals import Persistent
from Globals import DTMLFile
from Globals import MessageDialog

from Acquisition import Implicit

from OFS.SimpleItem import Item
from OFS.Folder import Folder

from DateTime import DateTime

#from IOnlineCourse import IOnlineCourse, IAssignment
import sys, os, string, whrandom


################## Assignment #######################



class Assignment(
    Implicit,
    RoleManager,
    Persistent,
    Item
    ):
    """Assignment object."""

#    __implements__ = IAssignment

    meta_type = 'Assignment'

    security = ClassSecurityInfo()

    manage_options = (
	{'label': 'Properties','action':'editAssignmentForm'},
	)


    def __init__(self, id, title='', assigned_date='', due_date='', possible_points='',
		 references='', intro='', description=''):
	self.id = id
	self.title = title
	self._assigned_date = DateTime(assigned_date)
	self._due_date = DateTime(due_date)
	self._possible_points = int(possible_points)
	self._references = references
	self._intro = intro
	self._description = description

    security.declareProtected('Change OnlineCourse', 'addAssignmentForm')
    addAssignmentForm = DTMLFile('dtml/add/addAssignment', globals())

    security.declareProtected('Change OnlineCourse', 'addAssignment')
    def addAssignment(self, title='', assigned_date='', due_date='', possible_points='',
		      references='', intro='', description='', REQUEST=None):
	"""Web interface to add a Assignment instance to an OnlineCourse folder."""
	id = string.strip(title)
	id = string.lower(id)
	id = string.replace(id,' ', '_')
	
	self._setObject(id, Assignment(id, title, assigned_date, due_date, possible_points,
				       references, intro, description))
	if REQUEST is not None:
	    if hasattr(REQUEST, 'idx'):
		return self.index_html(self, REQUEST)
	    else:
		return self.manage_main(self, REQUEST)
    security.declarePrivate('_editAssignmentForm')
    _editAssignmentForm = DTMLFile('dtml/edit/editAssignment', globals())

    security.declareProtected('Change OnlineCourse', 'editAssignmentForm')
    def editAssignmentForm(self, REQUEST=None, management_view='', manage_tabs_message=''):
	"""Form to edit Assignment Instance."""
	REQUEST['title'] = self.title
	REQUEST['assigned_date'] = self._assigned_date.strftime('%m/%d/%Y')
	REQUEST['due_date'] = self._due_date.strftime('%m/%d/%Y')
	REQUEST['possible_points'] = str(self._possible_points)
	REQUEST['references'] = self._references
	REQUEST['intro'] = self._intro
	REQUEST['description'] = self._description
	return self._editAssignmentForm(self, REQUEST,
					  management_view='Properties',
					  manage_tabs_message=manage_tabs_message)


    security.declareProtected('Change OnlineCourse', 'editAssignment')
    def editAssignment(self, assigned_date='', due_date='', possible_points='',
		       references='', intro='', description='', REQUEST=None):
	"""Edit Assignment Instance"""
	self._assigned_date = DateTime(assigned_date)
	self._due_date = DateTime(due_date)
	self._possible_points = int(possible_points)
	self._references = references
	self._intro = intro
	self._description = description
	if REQUEST is not None:
	    return self.editAssignmentForm(REQUEST,
					   manage_tabs_message='Assignment Changed.')
	return None

    security.declareProtected('View OnlineCourse', 'getAssignedDate')
    def getAssignedDate(self):
	"""Return the Assigned Date"""
	return self._assigned_date

    security.declareProtected('View OnlineCourse', 'getAssignedDateAsString')
    def getAssignedDateAsString(self):
	"""Return the Assigned Date as a string"""
	return self._assigned_date.strftime('%m/%d/%Y')

    security.declareProtected('View OnlineCourse', 'getDueDate')
    def getDueDate(self):
	"""Return the Assignmenst Due Date"""
	return self._due_date

    security.declareProtected('View OnlineCourse', 'getDueDateAsString')
    def getDueDateAsString(self):
	"""Return the Assignmenst Due Date"""
	return self._due_date.strftime('%m/%d/%Y')

    security.declareProtected('View OnlineCourse', 'getPossiblePoints')
    def getPossiblePoints(self):
	"""Return the Possible Points"""
	return self._possible_points

    security.declareProtected('View OnlineCourse', 'getReferences')
    def getReferences(self):
	"""Return References"""
	return self._references

    security.declareProtected('View OnlineCourse', 'getIntro')
    def getIntro(self):
	"""Return Introduction"""
	return self._intro

    security.declareProtected('View OnlineCourse', 'getDescription')
    def getDescription(self):
	"""Return Description"""
	return self._description

    security.declareProtected('View OnlineCourse', 'okToShow')
    def okToShow(self):
	"""Assignment ok to show?"""
	if (self._assigned_date.lessThanEqualTo(DateTime('US/Pacific'))
	    and
	    DateTime('US/Pacific').lessThanEqualTo(self._due_date)):
	    return 1
	else:
	    return 0	

    security.declareProtected('View OnlineCourse', 'index_html')
    index_html =  DTMLFile('dtml/display/indexAssignment', globals())

    security.setPermissionDefault('View OnlineCourse', ('Manager', 'Instructor', 'Student'))
    security.setPermissionDefault('Change OnlineCourse', ('Manager', 'Instructor'))

InitializeClass(Assignment)


################## AssignmentsFolder #######################

class AssignmentsFolder(
    Implicit,
    RoleManager,
    Persistent,
    Folder
    ):
    """Container class for assignements."""

#    __implements__ = IAssignmentsFolder

    meta_type='Assignments Folder'

    security=ClassSecurityInfo()

    def all_meta_types(self):
	all = AssignmentsFolder.inheritedAttribute('all_meta_types')(self)
	return ({'name':'Assignment', 'action': 'addAssignmentForm'},
		)

    addAssignmentForm = Assignment.addAssignmentForm
    addAssignment = Assignment.addAssignment

    manage_options = (
	{'label': 'Contents', 'action': 'manage_main'},
	)

    security.declareProtected('Change OnlineCourse', 'manage_main')
    security.declareProtected('Change OnlineCourse', 'manage_workspace')
    manage_workspace = manage_main = DTMLFile('dtml/display/mainAssignmentsFolder', globals())

    security.declarePrivate('addAssignmentsFolder')
    def addAssignmentsFolder(self, id, title='', REQUEST=None):
	"""Web interface to add an AssignmentsFolder instance to a folder."""

	self._setObject(id, AssignmentsFolder(id, title))
	if REQUEST is not None:
	    return self.manage_main(self, REQUEST)

    def __init__(self, id, title=''):
	self.id = id
	self.title = title

    security.declarePrivate('_editAssignmentsFolderForm')
    _editAssignmentsFolderForm = DTMLFile('dtml/edit/editAssignmentsFolder', globals())

    security.declareProtected('Change OnlineCourse', 'editAssignmentsFolderForm')
    def editAssignmentsFolderForm(self, REQUEST=None, management_view='', manage_tabs_message=''):
	"""Form to edit AssignmentsFolder Instance."""
	return self._editAssignmentsFolderForm(self, REQUEST,
					  management_view='Properties',
					  manage_tabs_message=manage_tabs_message)

    security.declareProtected('Change OnlineCourse', 'editAssignmentsFolder')
    def editAssignmentsFolder(self, title='', REQUEST=None):
	"""Edit AssignmentsFolder Instance"""
	self.title = title
	if REQUEST is not None:
	    return self.editAssignmentsFolderForm(REQUEST,
					     manage_tabs_message='Online Course Changed.')
	return None

    security.declareProtected('View OnlineCourse', 'index_html')
    index_html =  DTMLFile('dtml/display/indexAssignmentsFolder', globals())

    security.setPermissionDefault('View OnlineCourse', ('Manager', 'Instructor', 'Student'))
    security.setPermissionDefault('Change OnlineCourse', ('Manager', 'Instructor'))

InitializeClass(AssignmentsFolder)

