# OnlineCourse Product for Zope
# Copyright (C) 2001 Maurice Davis <mjd@networklogic.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# See file COPYING.txt
#
#
__doc__ = """ChatSessions Implementation for OnlineCourse Product."""

__version__ = '0.1.0'

from AccessControl import ClassSecurityInfo
from AccessControl.Role import RoleManager

from Globals import InitializeClass
from Globals import Persistent
from Globals import DTMLFile
from Globals import MessageDialog

from Acquisition import Implicit

from OFS.SimpleItem import Item
from OFS.Folder import Folder

from DateTime import DateTime

from Products.ZRTChat.ZRTChat import ZRTChat

#from IOnlineCourse import IOnlineCourse, IMessage
import sys, os, string, whrandom


################## ChatSession #######################

class ChatSession(
    ZRTChat
    ):
    """Chat Session object."""

#    __implements__ = IChatSession

    meta_type = 'Chat Session'

    security = ClassSecurityInfo()

    manage_options = (
	{'label': 'Properties','action':'editChatSessionForm'},
	)


    def __init__(self, id, topic='', start_date='', end_date=''):
	ZRTChat.__init__(self, id)
	self.topic = topic
	self._start_date = start_date
	self._end_date = end_date

    security.declareProtected('Change OnlineCourse', 'addChatSessionForm')
    addChatSessionForm = DTMLFile('dtml/add/addChatSession', globals())

    security.declareProtected('Change OnlineCourse', 'addChatSession')
    def addChatSession(self, topic='', start_date='', start_time='', end_date='', end_time='',
		       REQUEST=None):
	"""Web interface to add a ChatSession instance to an OnlineCourse folder."""
	id = string.strip(topic)
	id = string.lower(id)
	id = string.replace(id,' ', '_')
	start_date = DateTime(start_date + ' ' + start_time + ' US/Pacific')
	end_date = DateTime(end_date + ' ' + end_time + ' US/Pacific')
	self._setObject(id, ChatSession(id, topic, start_date, end_date))
	if REQUEST is not None:
	    if hasattr(REQUEST, 'idx'):
		return self.index_html(self, REQUEST)
	    else:
		return self.manage_main(self, REQUEST)

    security.declarePrivate('_editChatSessionForm')
    _editChatSessionForm = DTMLFile('dtml/edit/editChatSession', globals())

    security.declareProtected('Change OnlineCourse', 'editChatSessionForm')
    def editChatSessionForm(self, REQUEST=None, management_view='', manage_tabs_message=''):
	"""Form to edit ChatSession Instance."""
	REQUEST['topic'] = self.topic
	REQUEST['start_date'] = self._start_date.strftime('%m/%d/%Y')
	REQUEST['start_time'] = self._start_date.strftime('%I:%M%p %Z')
	REQUEST['end_date'] = self._end_date.strftime('%m/%d/%Y')
	REQUEST['end_time'] = self._end_date.strftime('%I:%M%p %Z')
	return self._editChatSessionForm(self, REQUEST,
					  management_view='Properties',
					  manage_tabs_message=manage_tabs_message)

    security.declareProtected('Change OnlineCourse', 'editChatSession')
    def editChatSession(self, topic='', start_date='', start_time='', end_date='', end_time='',
			REQUEST=None):
	"""Edit ChatSession Instance"""
	self.topic = topic
	self._start_date = DateTime(start_date + ' ' +start_time)
	self._end_date = DateTime(end_date + ' ' + end_time)
	if REQUEST is not None:
	    return self.editChatSessionForm(REQUEST,
					manage_tabs_message='Chat Session Changed.')
	return None

    security.declareProtected('Change OnlineCourse', 'manage_workspace')
    security.declareProtected('Change OnlineCourse', 'manage_main')
    manage_workspace = manage_main = editChatSessionForm

    security.declareProtected('View OnlineCourse', 'index_html')
    index_html = DTMLFile('dtml/display/indexChatSession', globals())

    security.declareProtected('View OnlineCourse', 'chat_header')
    chat_header = DTMLFile('dtml/display/chat_header', globals())

    security.declareProtected('View OnlineCourse', 'chat_frame')
    chat_frame = DTMLFile('dtml/display/chat_frame', globals())

    security.declareProtected('View OnlineCourse', 'form_frame')
    form_frame = DTMLFile('dtml/display/form_frame', globals())

    security.declareProtected('Change OnlineCourse', 'getTopic')
    def getTopic(self):
	"""Return ChatSession Topic."""
	return self.topic

    security.declareProtected('View OnlineCourse', 'getStartDateTime')
    def getStartDateTime(self):
	"""Return the Start Date and Time."""
	return self._start_date

    security.declareProtected('View OnlineCourse', 'getStartDateTimeAsString')
    def getStartDateTimeAsString(self):
	"""Return the Start Date and Time as a string."""
	return self._start_date.strftime('%m/%d/%Y %I:%M%p %Z')

    security.declareProtected('View OnlineCourse', 'getEndDateTime')
    def getEndDateTime(self):
	"""Return the Ending Date and Time."""
	return self._end_date

    security.declareProtected('View OnlineCourse', 'getEndDateTimeAsString')
    def getEndDateTimeAsString(self):
	"""Return the Ending Date and Time as a string."""
	return self._end_date.strftime('%m/%d/%Y %I:%M%p %Z')

    security.declareProtected('View OnlineCourse', 'okToPost')
    def okToPost(self):
	"""Allow posting to class chat session?"""
	if ((self.getStartDateTime().timeTime() - DateTime('US/Pacific').timeTime()
	    <= 900.0)
	    and
	    (DateTime('US/Pacific').timeTime() - self.getEndDateTime().timeTime() 
	     <= 600.0)):
	    return 1
	else:
	    return 0

InitializeClass(ChatSession)


################## ChatSessionsFolder #######################

class ChatSessionsFolder(
    Implicit,
    RoleManager,
    Persistent,
    Folder
    ):
    """Container class for chat sessions."""

#    __implements__ = IChatSessionsFolder

    meta_type='Chat Sessions Folder'

    security=ClassSecurityInfo()

    def all_meta_types(self):
	all = ChatSessionsFolder.inheritedAttribute('all_meta_types')(self)
	return ({'name':'ChatSession', 'action': 'addChatSessionForm'},
		)

    addChatSessionForm = ChatSession.addChatSessionForm
    addChatSession = ChatSession.addChatSession

    manage_options = (
	{'label': 'Contents', 'action': 'manage_main'},
	)

    security.declareProtected('Change OnlineCourse', 'addChatSessionsFolder')
    def addChatSessionsFolder(self, id, title='', REQUEST=None):
	"""Web interface to add an ChatSessionsFolder instance to a folder."""

	self._setObject(id, ChatSessionsFolder(id, title))
	if REQUEST is not None:
	    return self.manage_main(self, REQUEST)

    def __init__(self, id, title=''):
	self.id = id
	self.title = title

    security.declareProtected('Change OnlineCourse', '_editChatSessionsFolderForm')
    _editChatSessionsFolderForm = DTMLFile('dtml/edit/editChatSessionsFolder', globals())

    security.declareProtected('Change OnlineCourse', 'editChatSessionsFolderForm')
    def editChatSessionsFolderForm(self, REQUEST=None, management_view='', manage_tabs_message=''):
	"""Form to edit ChatSessionsFolder Instance."""
	return self._editChatSessionsFolderForm(self, REQUEST,
					  management_view='Properties',
					  manage_tabs_message=manage_tabs_message)

    security.declareProtected('Change OnlineCourse', 'manage_workspace')
    security.declareProtected('Change OnlineCourse', 'manage_main')
    manage_workspace = manage_main = DTMLFile('dtml/display/mainChatSessionsFolder', globals())

    security.declareProtected('Change OnlineCourse', 'editChatSessionsFolder')
    def editChatSessionsFolder(self, title='', REQUEST=None):
	"""Edit ChatSessionsFolder Instance"""
	self.title = title
	if REQUEST is not None:
	    return self.editChatSessionsFolderForm(REQUEST,
					     manage_tabs_message='Online Course Changed.')
	return None

    security.declareProtected('View OnlineCourse', 'index_html')
    index_html =  DTMLFile('dtml/display/indexChatSessionsFolder', globals())

    security.setPermissionDefault('View OnlineCourse', ('Manager', 'Instructor', 'Student'))
    security.setPermissionDefault('Change OnlineCourse', ('Manager', 'Instructor'))

InitializeClass(ChatSessionsFolder)

