# OnlineCourse Product for Zope
# Copyright (C) 2001 Maurice Davis <mjd@networklogic.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# See file COPYING.txt
#
#
__doc__ = """HomeWork Implementation for OnlineCourse Product."""

__version__ = '0.1.0'

from AccessControl import ClassSecurityInfo
from AccessControl.Role import RoleManager

from Globals import InitializeClass
from Globals import Persistent
from Globals import DTMLFile
from Globals import MessageDialog

from Acquisition import Implicit

from OFS.SimpleItem import Item
from OFS.Folder import Folder

from DateTime import DateTime

#from IOnlineCourse import IOnlineCourse, IMessage
import sys, os, string, whrandom


################## HomeWork #######################



class HomeWork(
    Implicit,
    RoleManager,
    Persistent,
    Item
    ):
    """HomeWork object."""

#    __implements__ = IHomeWork

    meta_type = 'HomeWork'

    security = ClassSecurityInfo()

    manage_options = (
	{'label': 'Properties','action':'editHomeWorkForm'},
	)


    def __init__(self, id, student_id, assignment_id, possible_points, status='In Progress', answer=''):
	self.id = id
	self._student_id = student_id
	self._assignment_id = assignment_id
	self._possible_points = possible_points
	self._status = status
	self._completion_date = None
	self._answer = answer

    security.declareProtected('Change OnlineCourse', 'addHomeWorkForm')
    addHomeWorkForm = DTMLFile('dtml/add/addHomeWork', globals())

    security.declareProtected('Change OnlineCourse', 'addHomeWork')
    def addHomeWork(self, assignment_id, status='In Progress', answer='', REQUEST=None):
	"""Web interface to add a HomeWork instance to an OnlineCourse folder."""
	student_id = self.getMemberID()
	id = assignment_id + '_' + student_id
	possible_points = self.getAssignment(assignment_id).getPossiblePoints()
	self._setObject(id, HomeWork(id, student_id, assignment_id, possible_points, status, answer))
	if REQUEST is not None:
	    if hasattr(REQUEST, 'idx'):
		return self.index_html(self, REQUEST)
	    else:
		return self.manage_main(self, REQUEST)

    security.declarePrivate('_editHomeWorkForm')
    _editHomeWorkForm = DTMLFile('dtml/edit/editHomeWork', globals())

    security.declareProtected('View OnlineCourse', 'editHomeWorkForm')
    def editHomeWorkForm(self, REQUEST=None, management_view='', manage_tabs_message=''):
	"""Form to edit HomeWork Instance."""
	return self._editHomeWorkForm(self, REQUEST,
					  management_view='Properties',
					  manage_tabs_message=manage_tabs_message)

    security.declareProtected('View OnlineCourse', 'editHomeWork')
    def editHomeWork(self, status='In Progress', answer='', REQUEST=None):
	"""Edit HomeWork Instance"""
	if status == 'Turn in Home Work':
	    self._status = 'Completed'
	    self._completion_date = DateTime('US/Pacific')
	else:
	    self._status = 'In Progress'
	    self._completion_date = None
	self._answer = answer
	if REQUEST is not None and self._status == 'In Progress':
	    return self.editHomeWorkForm(REQUEST,
					manage_tabs_message='Home Work Changed.')
	elif REQUEST is not None and hasattr(REQUEST, 'idx'):
	    return REQUEST['RESPONSE'].redirect(REQUEST['URL2']+'/')
	elif REQUEST is not None:
	    return REQUEST['RESPONSE'].redirect(REQUEST['URL1']+'/manage_main')
	return None

    security.declareProtected('Change OnlineCourse', 'correctHomeWorkForm')
    correctHomeWorkForm = DTMLFile('dtml/edit/correctHomeWorkForm', globals())

    security.declareProtected('Change OnlineCourse', 'correctHomeWork')
    def correctHomeWork(self, grade='', status='Graded', REQUEST=None):
	"""Grade HomeWork Instance"""
	self._status = 'Graded'
	self._graded_points = int(grade)
	if REQUEST is not None and hasattr(REQUEST, 'idx'):
	    return REQUEST['RESPONSE'].redirect(REQUEST['URL2']+'/')
	elif REQUEST is not None:
	    return REQUEST['RESPONSE'].redirect(REQUEST['URL1']+'/manage_main')
	return None

    security.declareProtected('View OnlineCourse', 'getStudentID')
    def getStudentID(self):
	"""Return the student id for this homework."""
	return self._student_id

    security.declareProtected('View OnlineCourse', 'getAssignmentID')
    def getAssignmentID(self):
	"""Return the assignment id for this homework"""
	return self._assignment_id

    security.declareProtected('View OnlineCourse', 'getStatus')
    def getStatus(self):
	"""Return the homeworks status"""
	return self._status

    security.declareProtected('View OnlineCourse', 'getAssignmentTitle')
    def getAssignmentTitle(self):
	"""Return the homework question"""
	assignment_ob = self.assignments._getOb(self._assignment_id)
	return assignment_ob.title

    security.declareProtected('View OnlineCourse', 'getAssignmentDescription')
    def getAssignmentDescription(self):
	"""Return the homework question"""
	assignment_ob = self.assignments._getOb(self._assignment_id)
	return assignment_ob._description

    security.declareProtected('View OnlineCourse', 'getAssignmentDueDate')
    def getAssignmentDueDate(self):
	"""Return the homework assigned due date"""
	assignment_ob = self.assignments._getOb(self._assignment_id)
	return assignment_ob._due_date.strftime('%m/%d/%Y')

    security.declareProtected('View OnlineCourse', 'getCompletionDateAsString')
    def getCompletionDateAsString(self):
	"""Return the homework completion date"""
	if self._completion_date == None:
	    return 'In Progress'
	else:
	    return 'Completed ' + self._completion_date.strftime('%m/%d/%Y')

    security.declareProtected('View OnlineCourse', 'getAnswer')
    def getAnswer(self):
	"""Return the homeworks answer"""
	return self._answer

    security.declareProtected('View OnlineCourse', 'getPP')
    def getPP(self):
	"""Return the homeworks possible points"""
	return self._possible_points

    security.declareProtected('View OnlineCourse', 'getGrade')
    def getGrade(self):
	"""Return the homeworks possible points"""
	if hasattr(self, '_graded_points'):
	    return str(self._graded_points)
	else:
	    return ''

    security.declareProtected('View OnlineCourse', 'index_html')
    index_html =  DTMLFile('dtml/display/indexHomeWork', globals())

    security.setPermissionDefault('View OnlineCourse', ('Manager', 'Instructor', 'Student'))
    security.setPermissionDefault('Change OnlineCourse', ('Manager', 'Instructor'))

InitializeClass(HomeWork)


################## HomeWorkFolder #######################

class HomeWorkFolder(
    Implicit,
    RoleManager,
    Persistent,
    Folder
    ):
    """Container class for home work."""

#    __implements__ = IHomeWorkFolder

    meta_type='HomeWork Folder'

    security=ClassSecurityInfo()

    def all_meta_types(self):
	all = HomeWorkFolder.inheritedAttribute('all_meta_types')(self)
	return ({'name':'HomeWork', 'action': 'addHomeWorkForm'},
		)

    addHomeWorkForm = HomeWork.addHomeWorkForm
    addHomeWork = HomeWork.addHomeWork

    manage_options = (
	{'label': 'Contents', 'action': 'manage_main'},
	)

    security.declareProtected('Change OnlineCourse', 'manage_workspace')
    security.declareProtected('Change OnlineCourse', 'manage_main')
    manage_workspace = manage_main = DTMLFile('dtml/display/mainHomeWorkFolder', globals())

    security.declareProtected('Change OnlineCourse', 'addHomeWorkFolder')
    def addHomeWorkFolder(self, id, title='', REQUEST=None):
	"""Web interface to add an HomeWorkFolder instance to a folder."""

	self._setObject(id, HomeWorkFolder(id, title))
	if REQUEST is not None:
	    return self.manage_main(self, REQUEST)

    def __init__(self, id, title=''):
	self.id = id
	self.title = title

    security.declarePrivate('_editHomeWorkFolderForm')
    _editHomeWorkFolderForm = DTMLFile('dtml/edit/editHomeWorkFolder', globals())

    security.declareProtected('Change OnlineCourse', 'editHomeWorkFolderForm')
    def editHomeWorkFolderForm(self, REQUEST=None, management_view='', manage_tabs_message=''):
	"""Form to edit HomeWorkFolder Instance."""
	return self._editHomeWorkFolderForm(self, REQUEST,
					  management_view='Properties',
					  manage_tabs_message=manage_tabs_message)

    security.declareProtected('Change OnlineCourse', 'editHomeWorkFolder')
    def editHomeWorkFolder(self, title='', REQUEST=None):
	"""Edit HomeWorkFolder Instance"""
	self.title = title
	if REQUEST is not None:
	    return self.editHomeWorkFolderForm(REQUEST,
					     manage_tabs_message='Online Course Changed.')
	return None

    security.declareProtected('View OnlineCourse', 'getAssignmentIDOptions')
    def getAssignmentIDOptions(self):
	"""Return the homeworks status"""
	option_string = ''
	assignments_list = self.assignments.objectItems()
	i = 1
	for (assign_id, assign_ob) in assignments_list:
	    if assign_ob.okToShow():
		if i == 1:
		    option_string = (option_string + 
				     '<option value="' + assign_id + '" selected>' + assign_ob.title)
		else:
		    option_string = (option_string + '<option value="' + assign_id + '">' + assign_ob.title)
		i = i + 1
				 
	return option_string

    security.declareProtected('View OnlineCourse', 'getAssignment')
    def getAssignment(self, assignment_id):
	"""Get Assignment object from AssignmentID"""
	assignments_list = self.assignments.objectItems()
	for (assign_id, assign_ob) in assignments_list:
	    if assign_id == assignment_id:
		return assign_ob
	return None

    security.declareProtected('View OnlineCourse', 'index_html')
    index_html =  DTMLFile('dtml/display/indexHomeWorkFolder', globals())

    security.setPermissionDefault('View OnlineCourse', ('Manager', 'Instructor', 'Student'))
    security.setPermissionDefault('Change OnlineCourse', ('Manager', 'Instructor'))

InitializeClass(HomeWorkFolder)

