# OnlineCourse Product for Zope
# Copyright (C) 2001 Maurice Davis <mjd@networklogic.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# See file COPYING.txt
#
#
__doc__ = """Messages Implementation for OnlineCourse Product."""

__version__ = '0.1.0'

from AccessControl import ClassSecurityInfo
from AccessControl.Role import RoleManager
from AccessControl import getSecurityManager

from Globals import InitializeClass
from Globals import Persistent
from Globals import DTMLFile
from Globals import MessageDialog

from Acquisition import Implicit

from OFS.SimpleItem import Item
from OFS.Folder import Folder

from DateTime import DateTime

#from IOnlineCourse import IOnlineCourse, IMessage
import sys, os, string, whrandom


################## Message #######################

class Message(
    Implicit,
    RoleManager,
    Persistent,
    Item
    ):
    """Message object."""

#    __implements__ = IMessage

    meta_type = 'Message'

    security = ClassSecurityInfo()
    security.declareObjectProtected('View OnlineCourse')

    manage_options = (
	{'label': 'Properties','action':'editMessageForm'},
	{'label': 'View',       'action': 'index_html'},
	) + RoleManager.manage_options + Item.manage_options


    def __init__(self, id, title='', date_time=None, sender_id='', sender='',
		 recipient='', visibility='Private', content=''):
	self.id = id
	self.title = title
	self._date_time = date_time
	self._sender_id = sender_id
	self._sender = sender
	self._recipient = recipient
	self._visibility = visibility
	self._content = content

    security.declareProtected('Change OnlineCourse', 'addMessageForm')
    addMessageForm = DTMLFile('dtml/add/addMessage', globals())

    security.declareProtected('Change OnlineCourse', 'addMessage')
    def addMessage(self, title='', sender_id='', sender_full='', recipient='', visibility='Private',
		      content='', REQUEST=None):
	"""Web interface to add a Message instance to an OnlineCourse folder."""
	date_time = DateTime()
	id = sender_id + '_' + date_time.strftime('%m-%d-%Y-%I-%M-%S%p')

	self._setObject(id, Message(id, title, date_time, sender_id, sender_full,
				    recipient, visibility, content))
	if REQUEST is not None:
	    if hasattr(REQUEST, 'idx'):
		return self.index_html(self, REQUEST)
	    else:
		return self.manage_main(self, REQUEST)

    security.declarePrivate('_editMessageForm')
    _editMessageForm = DTMLFile('dtml/edit/editMessage', globals())

    security.declareProtected('Change OnlineCourse', 'editMessageForm')
    def editMessageForm(self, REQUEST=None, management_view='', manage_tabs_message=''):
	"""Form to edit Message Instance."""
	REQUEST['title'] = self.title
	REQUEST['sender'] = self._sender
	REQUEST['recipient'] = self._recipient
	REQUEST['visibility'] = self._visibility
	REQUEST['content'] = self._content
	return self._editMessageForm(self, REQUEST,
					  management_view='Properties',
					  manage_tabs_message=manage_tabs_message)


    security.declareProtected('Change OnlineCourse', 'editMessage')
    def editMessage(self, title='', sender=None, recipient=None, visibility='Private',
		    content='', REQUEST=None):
	"""Edit Message Instance"""
	self._title = title
	self._sender = sender
	self._recipient = recipient
	self._visibility = visibility
	self._content = content
	if REQUEST is not None:
	    return self.editMessageForm(REQUEST,
					manage_tabs_message='Course Member Changed.')
	return None

    security.declareProtected('View OnlineCourse', 'manage_main')
    security.declareProtected('View OnlineCourse', 'manage_workspace')
    manage_workspace = manage_main = DTMLFile('dtml/display/mainMessage', globals())

    security.declareProtected('View OnlineCourse', 'index_html')
    index_html = DTMLFile('dtml/display/indexMessage', globals())

    security.declareProtected('Change OnlineCourse', 'manage_workspace')
    manage_workspace = DTMLFile('dtml/display/displayMessage', globals())

    security.declareProtected('View OnlineCourse', 'getSubject')
    def getSubject(self):
	"""Return the message subject (title)"""
	return self.title

    security.declareProtected('View OnlineCourse', 'getSenderID')
    def getSenderID(self):
	"""Return the message sender"""
	return self._sender_id

    security.declareProtected('View OnlineCourse', 'getSender')
    def getSender(self):
	"""Return the message sender"""
	return self._sender

    security.declareProtected('View OnlineCourse', 'getRecipientID')
    def getRecipientID(self):
	"""Return the message addresse(s)"""
	return self._recipient

    security.declareProtected('View OnlineCourse', 'getRecipientFullName')
    def getRecipientFullName(self):
	"""Return the message addresse(s)"""
	for (mid, mob) in self.member_info.objectItems('Member'):
	    if self._recipient == mob.id:
		return mob.getFullName()
	else:
	    return 'All'

    security.declareProtected('View OnlineCourse', 'getRecipientFirstLast')
    def getRecipientFirstLast(self):
	"""Return the message addresse(s)"""
	for (mid, mob) in self.member_info.objectItems('Member'):
	    if self._recipient == mob.id:
		return mob.id
	else:
	    return 'All'

    security.declareProtected('View OnlineCourse', 'getDate')
    def getDate(self):
	"""Return the message addresse(s)"""
	return self._date_time.strftime('%m-%d-%Y %I:%M%p')

    security.declareProtected('View OnlineCourse', 'getVisibility')
    def getVisibility(self):
	"""Return the message visibility"""
	return self._visibility

    security.declareProtected('View OnlineCourse', 'getContent')
    def getContent(self):
	"""Return the message content"""
	return self._content

    security.declareProtected('View OnlineCourse', 'okToShow')
    def okToShow(self):
	"""Don't display private messages not to or from the member."""
	if self.memberIsInstructor():
	    return 1
	elif self.getVisibility() == 'Public':
	    return 1
	elif self.getVisibility() == 'Private':
	    if self.getSenderID() == self.getMemberID() or self.getRecipientID() == self.getMemberID():
		return 1
	    else:
		return 0
	else:
	    return 0

    security.setPermissionDefault('View OnlineCourse', ('Manager', 'Instructor', 'Student'))
    security.setPermissionDefault('Change OnlineCourse', ('Manager', 'Instructor'))

InitializeClass(Message)


################## MessagesFolder #######################

class MessagesFolder(
    Implicit,
    RoleManager,
    Persistent,
    Folder
    ):
    """Container class for messages."""

#    __implements__ = IMessagesFolder

    meta_type='Messages Folder'

    security=ClassSecurityInfo()

    def all_meta_types(self):
	all = MessagesFolder.inheritedAttribute('all_meta_types')(self)
	return ({'name':'Message', 'action': 'addMessageForm'},
		)

    addMessageForm = Message.addMessageForm
    addMessage = Message.addMessage

    manage_options = (
	{'label': 'Contents', 'action': 'manage_main'},
	)

    security.declareProtected('Change OnlineCourse', 'manage_workspace')
    security.declareProtected('Change OnlineCourse', 'manage_main')
    manage_workspace = manage_main = DTMLFile('dtml/display/mainMessagesFolder', globals())

    security.declareProtected('Change OnlineCourse', 'addMessagesFolder')
    def addMessagesFolder(self, id, title='', REQUEST=None):
	"""Web interface to add an MessagesFolder instance to a folder."""

	self._setObject(id, MessagesFolder(id, title))
	if REQUEST is not None:
	    return self.manage_main(self, REQUEST)

    def __init__(self, id, title=''):
	self.id = id
	self.title = title

    security.declarePrivate('Change OnlineCourse', '_editMessagesFolderForm')
    _editMessagesFolderForm = DTMLFile('dtml/edit/editMessagesFolder', globals())

    security.declareProtected('Change OnlineCourse', 'editMessagesFolderForm')
    def editMessagesFolderForm(self, REQUEST=None, management_view='', manage_tabs_message=''):
	"""Form to edit MessagesFolder Instance."""
	return self._editMessagesFolderForm(self, REQUEST,
					  management_view='Properties',
					  manage_tabs_message=manage_tabs_message)

    security.declareProtected('Change OnlineCourse', 'editMessagesFolder')
    def editMessagesFolder(self, title='', REQUEST=None):
	"""Edit MessagesFolder Instance"""
	self.title = title
	if REQUEST is not None:
	    return self.editMessagesFolderForm(REQUEST,
					     manage_tabs_message='Online Course Changed.')
	return None

    security.declareProtected('Change OnlineCourse', 'getMembersAsOptions')
    def getMembersAsOptions(self):
	"""Return options list of members."""
	option_string = ''
	members_list = self.member_info.objectItems('Member')
	for (member_id, member_ob) in members_list:
	    option_string = option_string + '<option value="' + member_id + '">' + member_ob.title
	return option_string

    security.declareProtected('View OnlineCourse', 'index_html')
    index_html =  DTMLFile('dtml/display/indexMessagesFolder', globals())

    security.setPermissionDefault('View OnlineCourse', ('Manager', 'Instructor', 'Student'))
    security.setPermissionDefault('Change OnlineCourse', ('Manager', 'Instructor'))

InitializeClass(MessagesFolder)

